////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.xfire;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.net.URL;
import java.util.Calendar;
import org.watto.android.Language;
import org.watto.android.R;
import org.watto.program.android.sync.xfire.array.ArrayResizer;
import org.watto.program.android.sync.xfire.datatype.AgeData;
import org.watto.program.android.sync.xfire.datatype.BiographyData;
import org.watto.program.android.sync.xfire.datatype.GamingStyleData;
import org.watto.program.android.sync.xfire.datatype.GenderData;
import org.watto.program.android.sync.xfire.datatype.InterestsData;
import org.watto.program.android.sync.xfire.datatype.JoinDateData;
import org.watto.program.android.sync.xfire.datatype.LocationData;
import org.watto.program.android.sync.xfire.datatype.OccupationData;
import org.watto.program.android.sync.xfire.datatype.StatusData;
import org.watto.program.android.sync.xfire.datatype.UsernameData;
import org.watto.program.android.sync.xfire.datatype.WebsiteData;
import android.content.ContentProviderOperation;
import android.content.ContentValues;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.provider.ContactsContract;
import android.provider.ContactsContract.CommonDataKinds.GroupMembership;
import android.provider.ContactsContract.CommonDataKinds.Nickname;
import android.provider.ContactsContract.CommonDataKinds.Phone;
import android.provider.ContactsContract.CommonDataKinds.Photo;
import android.provider.ContactsContract.CommonDataKinds.StructuredName;
import android.provider.ContactsContract.CommonDataKinds.StructuredPostal;
import android.provider.ContactsContract.Data;
import android.provider.ContactsContract.Groups;
import android.provider.ContactsContract.RawContacts;


/***********************************************************************************************
Helper class for storing data in the platform content providers
***********************************************************************************************/
public class ContactOperations {

  ContentValues content;
  ContentProviderOperation.Builder operationBuilder;
  BatchOperation batchOperation;
  Context context;
  boolean yield;
  long rawContactId = 0;
  int backReference;
  boolean newContact;


  /***********************************************************************************************
  Constructor
  ***********************************************************************************************/
  public ContactOperations(Context context, BatchOperation batchOperation){
    this.content = new ContentValues();
    this.yield = true;
    this.context = context;
    this.batchOperation = batchOperation;
  }


  /***********************************************************************************************
  Constructor for adding a new User
  ***********************************************************************************************/
  public ContactOperations(Context context, int userId, String accountName, BatchOperation batchOperation){
    this(context,batchOperation);
    this.backReference = batchOperation.size();
    this.newContact = true;

    content.put(RawContacts.SOURCE_ID,userId);
    content.put(RawContacts.ACCOUNT_TYPE,GlobalConstants.ACCOUNT_TYPE);
    content.put(RawContacts.ACCOUNT_NAME,accountName);

    this.operationBuilder = addInsertOperation(RawContacts.CONTENT_URI,true).withValues(content);
    batchOperation.add(operationBuilder.build());
  }


  /***********************************************************************************************
  Constructor for adding a new Group
  ***********************************************************************************************/
  public ContactOperations(Context context, int groupId, String accountName, BatchOperation batchOperation, boolean addingGroup){
    this(context,batchOperation);
    this.backReference = batchOperation.size();
    this.newContact = true;
  }


  /***********************************************************************************************
  Constructor for updating an existing User
  ***********************************************************************************************/
  public ContactOperations(Context context, long rawContactId, BatchOperation batchOperation){
    this(context,batchOperation);
    this.newContact = false;
    this.rawContactId = rawContactId;
  }




  /***********************************************************************************************
  Adds an update operation into the batch
  ***********************************************************************************************/
  public void addDeleteOperation(Uri uri){
    operationBuilder = addDeleteOperation(uri,yield);
    yield = false;
    batchOperation.add(operationBuilder.build());
  }



  /***********************************************************************************************
  Adds a "XFire" group
  @param title the name of the group
  @param notes a description of the group
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addGroup(String title,String notes){

    content.clear();
    if (title != null && !title.equals("") && notes != null && !notes.equals("")) {
      content.put(Groups.ACCOUNT_TYPE,GlobalConstants.ACCOUNT_TYPE);
      content.put(Groups.ACCOUNT_NAME,GlobalConstants.ACCOUNT_NAME);
      content.put(Groups.TITLE,title);
      content.put(Groups.NOTES,notes);
      content.put(Groups.GROUP_VISIBLE,"1");
      content.put(Groups.SHOULD_SYNC,"1");
    }

    if (content.size() > 0) {
      operationBuilder = addInsertOperation(addCallerIsSyncAdapterParameter(Groups.CONTENT_URI),yield);
      operationBuilder.withValues(content);

      yield = false;
      batchOperation.add(operationBuilder.build());
      // MUST execute here, so the group is created and can be referenced when adding users to the group
      batchOperation.execute();

    }

    return this;

  }


  /***********************************************************************************************
  Adds a contact to a group
  @param groupId the ID of the group to add the contact to
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addGroupMembership(long groupId){
    content.clear();
    if (groupId != -1) {
      content.put(Data.MIMETYPE,GroupMembership.CONTENT_ITEM_TYPE);
      content.put(GroupMembership.GROUP_ROW_ID,groupId);
    }

    if (content.size() > 0) {
      addInsertOperation();
    }
    return this;
  }



  /***********************************************************************************************
  Adds an insert operation into the batch
  ***********************************************************************************************/
  public void addInsertOperation(){

    if (!newContact) {
      content.put(Phone.RAW_CONTACT_ID,rawContactId);
    }

    operationBuilder = addInsertOperation(addCallerIsSyncAdapterParameter(Data.CONTENT_URI),yield);
    operationBuilder.withValues(content);

    if (newContact) {
      operationBuilder.withValueBackReference(Data.RAW_CONTACT_ID,backReference);
    }

    yield = false;
    batchOperation.add(operationBuilder.build());
  }
  
  
  /***********************************************************************************************
  Adds a country
  @param country the country
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addCountry(String country){
    content.clear();
    if (country != null && !country.equals("")) {
      content.put(StructuredPostal.MIMETYPE,StructuredPostal.CONTENT_ITEM_TYPE);
      content.put(StructuredPostal.FORMATTED_ADDRESS,country);
      content.put(StructuredPostal.TYPE,StructuredPostal.TYPE_CUSTOM);
      content.put(StructuredPostal.LABEL,context.getString(R.string.label_country));
      content.put(StructuredPostal.COUNTRY,country);
    }

    if (content.size() > 0) {
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds a location
  @param location the location data to add
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addLocation(String location){
    content.clear();
    if (location != null && !location.equals("")) {
      content.put(LocationData.DATA,location);
      content.put(LocationData.LABEL,context.getString(R.string.label_location));
      content.put(Data.MIMETYPE,LocationData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds a member since data
  @param memberSince the member since data to add
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addJoinDate(String memberSince){
    content.clear();
    if (memberSince != null && !memberSince.equals("")) {
      content.put(JoinDateData.DATA,memberSince);
      content.put(JoinDateData.LABEL,context.getString(R.string.label_joindate));
      content.put(Data.MIMETYPE,JoinDateData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }
  

  /***********************************************************************************************
  Adds a gender for a contact
  @param gender the gender data to add
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addGender(String gender){
    content.clear();
    if (gender != null && !gender.equals("")) {
      content.put(GenderData.DATA,gender);
      content.put(GenderData.LABEL,context.getString(R.string.label_gender));
      content.put(Data.MIMETYPE,GenderData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Adds a gaming style for a contact
  @param gamingStyle the gaming style to add
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addGamingStyle(String gamingStyle){
    content.clear();
    if (gamingStyle != null && !gamingStyle.equals("")) {
      content.put(GamingStyleData.DATA,gamingStyle);
      content.put(GamingStyleData.LABEL,context.getString(R.string.label_gamingstyle));
      content.put(Data.MIMETYPE,GamingStyleData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Adds a age for a contact
  @param age the age data to add
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addAge(int age){
    content.clear();
    if (age != 0 && age != -1) {
      content.put(AgeData.DATA,age);
      content.put(AgeData.LABEL,context.getString(R.string.label_age));
      content.put(Data.MIMETYPE,AgeData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Adds a occupation for a contact
  @param occupation the occupation to add
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addOccupation(String occupation){
    content.clear();
    if (occupation != null && !occupation.equals("")) {
      content.put(OccupationData.DATA,occupation);
      content.put(OccupationData.LABEL,context.getString(R.string.label_occupation));
      content.put(Data.MIMETYPE,OccupationData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Adds a interests for a contact
  @param interests the interests to add
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addInterests(String interests){
    content.clear();
    if (interests != null && !interests.equals("")) {
      content.put(InterestsData.DATA,interests);
      content.put(InterestsData.LABEL,context.getString(R.string.label_interests));
      content.put(Data.MIMETYPE,InterestsData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Adds a interests for a contact
  @param interests the interests to add
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addBiography(String biography){
    content.clear();
    if ( biography != null && ! biography.equals("")) {
      content.put(BiographyData.DATA, biography);
      content.put(BiographyData.LABEL,context.getString(R.string.label_biography));
      content.put(Data.MIMETYPE,BiographyData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }




  /***********************************************************************************************
  Adds the online status
  @param status the online status to add
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addStatus(String status){
    content.clear();
    if (status != null && !status.equals("")) {
      content.put(StatusData.DATA,status);
      content.put(StatusData.LABEL,context.getString(R.string.label_status));
      content.put(Data.MIMETYPE,StatusData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds a photo
  @param photoURL the URL to the photo
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addPhoto(URL photoURL){
    content.clear();

    if (photoURL != null && !photoURL.toString().equals("")) {
      // Download the photo into a byte[] BLOB (stored as a String)
      byte[] photoBytes = downloadPhoto(photoURL);

      // Only add a photo if the download was successful
      if (photoBytes != null && photoBytes.length > 0) {
        // store the date/time that this photo was retrieved, so its only grabbed once a day
        String timestamp = "" + Calendar.getInstance().getTimeInMillis();

        // Store the photo
        content.put(Photo.PHOTO,photoBytes);
        content.put(Photo.SYNC1,timestamp);
        content.put(Photo.MIMETYPE,Photo.CONTENT_ITEM_TYPE);
        addInsertOperation();
      }
    }
    return this;
  }




  /***********************************************************************************************
  Adds the real name of a user
  @param realName the real name of the user
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addRealName(String realName){
    content.clear();
    if (realName != null && !realName.equals("")) {
      content.put(StructuredName.DISPLAY_NAME,realName);
      content.put(StructuredName.MIMETYPE,StructuredName.CONTENT_ITEM_TYPE);
    }
    if (content.size() > 0) {
      addInsertOperation();
    }
    return this;
  }






  /***********************************************************************************************
  Adds an update operation into the batch
  ***********************************************************************************************/
  public void addUpdateOperation(Uri uri){
    operationBuilder = addUpdateOperation(uri,yield).withValues(content);
    yield = false;
    batchOperation.add(operationBuilder.build());
  }


  /***********************************************************************************************
  Adds the users username
  @param username the username on the backend server
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addUsername(String username){
    content.clear();
    if (username != null && !username.equals("")) {
      content.put(UsernameData.DATA,username);
      content.put(UsernameData.LABEL,context.getString(R.string.label_username));
      content.put(Data.MIMETYPE,UsernameData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds the username of a user
  @param nickname the username
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addNickname(String nickname){
    content.clear();
    if (nickname != null && !nickname.equals("")) {
      content.put(Nickname.NAME,nickname);
      content.put(Nickname.MIMETYPE,Nickname.CONTENT_ITEM_TYPE);
      content.put(Nickname.TYPE,Nickname.TYPE_CUSTOM);
      content.put(Nickname.LABEL,Language.get(R.string.label_nickname));
    }
    if (content.size() > 0) {
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds a website
  @param website the website to add
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addWebsite(String website){
    content.clear();
    if (website != null && !website.equals("")) {
      content.put(WebsiteData.DATA,website);
      content.put(WebsiteData.LABEL,context.getString(R.string.label_website));
      content.put(Data.MIMETYPE,WebsiteData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }




  /***********************************************************************************************
  Updates the users photo
  @param photoURL the new photo URL
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations deletePhoto(Uri contactUrl){
    addDeleteOperation(contactUrl);
    return this;
  }


  /***********************************************************************************************
  Downloads a photo from a <code>URL</code>. If the image is too large, it is resized.
  @param photoURL the URL to the photo
  @return the <code>byte</code> array of photo data (in JPEG format)
  ***********************************************************************************************/
  public byte[] downloadPhoto(URL photoURL){

    if (photoURL != null && !photoURL.toString().equals("")) {
      // Download the photo into a byte[] BLOB (stored as a String)
      byte[] photoBytes = new byte[1000];
      boolean photoSuccess = false;
      try {
        InputStream dataStream = photoURL.openStream();

        int readSize = 0;
        int receivedSize = dataStream.read(photoBytes,readSize,1000);

        while (receivedSize > 0) {
          // there is more data, so increase the array by another 1000 bytes and repeat the read
          readSize += receivedSize;
          photoBytes = ArrayResizer.resize(photoBytes,readSize + 1000);
          receivedSize = dataStream.read(photoBytes,readSize,1000);
        }

        dataStream.close();

        // resize to the actual size of the content
        photoBytes = ArrayResizer.resize(photoBytes,readSize);

        // Now check that the bytes are a valid JPEG file
        // (in case the URL was invalid or a 404 page was downloaded, etc)
        // (ie. only store the image if it is actually an image)
        Bitmap loadedJpg = BitmapFactory.decodeByteArray(photoBytes,0,readSize);
        if (loadedJpg != null) {
          // if the bitmap is larger than 150x150, resize it...
          int width = loadedJpg.getWidth();
          int height = loadedJpg.getHeight();
          if (width > 150 || height > 150) {
            width = 150;
            height = 150;
            // resize it
            loadedJpg = Bitmap.createScaledBitmap(loadedJpg,width,height,true);
            // get the new bytes for the resized image
            ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
            loadedJpg.compress(Bitmap.CompressFormat.JPEG,75,byteStream);
            photoBytes = byteStream.toByteArray();
          }
          photoSuccess = true;
        }
      }
      catch (Throwable t) {
        // Error downloading the photo
      }

      // Only add a photo if the download was successful
      if (photoSuccess) {
        return photoBytes;
      }

    }

    return null;
  }


  /***********************************************************************************************
  Formats a timestamp in a <code>Calendar</code> into a text <code>String</code>
  @param calendar the <code>Calendar</code> set to the timestamp
  @return the <code>String</code> representation of the timestamp
  ***********************************************************************************************/
  public String formatTimestamp(Calendar calendar){

    // Hour
    String timestamp = calendar.get(Calendar.HOUR) + ":";

    // Minute
    int minute = calendar.get(Calendar.MINUTE);
    if (minute < 10) {
      timestamp += "0";
    }
    timestamp += minute;

    // AM/PM
    if (calendar.get(Calendar.HOUR_OF_DAY) > 12) {
      timestamp += context.getString(R.string.time_pm) + " ";
    }
    else {
      timestamp += context.getString(R.string.time_am) + " ";
    }

    // Day
    int day = calendar.get(Calendar.DAY_OF_WEEK);
    if (day == Calendar.MONDAY) {
      timestamp += context.getString(R.string.day_monday) + " ";
    }
    else if (day == Calendar.TUESDAY) {
      timestamp += context.getString(R.string.day_tuesday) + " ";
    }
    else if (day == Calendar.WEDNESDAY) {
      timestamp += context.getString(R.string.day_wednesday) + " ";
    }
    else if (day == Calendar.THURSDAY) {
      timestamp += context.getString(R.string.day_thursday) + " ";
    }
    else if (day == Calendar.FRIDAY) {
      timestamp += context.getString(R.string.day_friday) + " ";
    }
    else if (day == Calendar.SATURDAY) {
      timestamp += context.getString(R.string.day_saturday) + " ";
    }
    else if (day == Calendar.SUNDAY) {
      timestamp += context.getString(R.string.day_sunday) + " ";
    }

    // Day number
    day = calendar.get(Calendar.DAY_OF_MONTH);
    timestamp += day;

    if (day == 1 || day == 21 || day == 31) {
      timestamp += context.getString(R.string.day_suffix_st) + " ";
    }
    else if (day == 2 || day == 22) {
      timestamp += context.getString(R.string.day_suffix_nd) + " ";
    }
    else if (day == 3 || day == 23) {
      timestamp += context.getString(R.string.day_suffix_rd) + " ";
    }
    else {
      timestamp += context.getString(R.string.day_suffix_th) + " ";
    }

    // Month
    int month = calendar.get(Calendar.MONTH);
    if (month == Calendar.JANUARY) {
      timestamp += context.getString(R.string.month_january) + " ";
    }
    else if (month == Calendar.FEBRUARY) {
      timestamp += context.getString(R.string.month_february) + " ";
    }
    else if (month == Calendar.MARCH) {
      timestamp += context.getString(R.string.month_march) + " ";
    }
    else if (month == Calendar.APRIL) {
      timestamp += context.getString(R.string.month_april) + " ";
    }
    else if (month == Calendar.MAY) {
      timestamp += context.getString(R.string.month_may) + " ";
    }
    else if (month == Calendar.JUNE) {
      timestamp += context.getString(R.string.month_june) + " ";
    }
    else if (month == Calendar.JULY) {
      timestamp += context.getString(R.string.month_july) + " ";
    }
    else if (month == Calendar.AUGUST) {
      timestamp += context.getString(R.string.month_august) + " ";
    }
    else if (month == Calendar.SEPTEMBER) {
      timestamp += context.getString(R.string.month_september) + " ";
    }
    else if (month == Calendar.OCTOBER) {
      timestamp += context.getString(R.string.month_october) + " ";
    }
    else if (month == Calendar.NOVEMBER) {
      timestamp += context.getString(R.string.month_november) + " ";
    }
    else if (month == Calendar.DECEMBER) {
      timestamp += context.getString(R.string.month_december) + " ";
    }

    // Year
    timestamp += calendar.get(Calendar.YEAR);

    return timestamp;
  }



  /***********************************************************************************************
  Updates the country
  @param country the new country
  @param oldCountry the old country
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateCountry(String country,String oldCountry,Uri contactUrl){
    content.clear();
    if (country != null && oldCountry != null && !country.equals(oldCountry)) {
      content.put(StructuredPostal.FORMATTED_ADDRESS,country);
      content.put(StructuredPostal.COUNTRY,country);
    }

    if (content.size() > 0) {
      addUpdateOperation(contactUrl);
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Updates a location
  @param location the new location data
  @param oldLocation the old location data
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateLocation(String location,String oldLocation,Uri contactUrl){
    if (location != null && oldLocation != null && !location.equals(oldLocation)) {
      content.clear();
      content.put(LocationData.DATA,location);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates a join date
  @param joinDate the new join date data
  @param oldJoinDate the old join date data
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateJoinDate(String joinDate,String oldJoinDate,Uri contactUrl){
    if (joinDate != null && oldJoinDate != null && !joinDate.equals(oldJoinDate)) {
      content.clear();
      content.put(JoinDateData.DATA,joinDate);
      addUpdateOperation(contactUrl);
    }
    return this;
  }



  /***********************************************************************************************
  Updates the online status
  @param status the new online status
  @param oldStatus the old online status
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateStatus(String status,String oldStatus,Uri contactUrl){
    if (status != null && oldStatus != null && !status.equals(oldStatus)) {
      content.clear();
      content.put(StatusData.DATA,status);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the users photo
  @param photoURL the new photo URL
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updatePhoto(URL photoURL,Uri contactUrl){
    if (photoURL != null && !photoURL.toString().equals("")) {
      content.clear();

      // Download the photo into a byte[] BLOB (stored as a String)
      byte[] photoBytes = downloadPhoto(photoURL);

      // Only update the photo if the download was successful
      if (photoBytes != null && photoBytes.length > 0) {
        // Store the photo
        content.put(Photo.PHOTO,photoBytes);
        addUpdateOperation(contactUrl);
      }

    }
    return this;
  }




  /***********************************************************************************************
  Updates the users real name
  @param realName the new name
  @param oldRealName the old name
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateRealName(String realName,String oldRealName,Uri contactUrl){
    content.clear();
    if (realName != null && oldRealName != null && !realName.equals(oldRealName)) {
      content.put(StructuredName.DISPLAY_NAME,realName);
    }
    if (content.size() > 0) {
      addUpdateOperation(contactUrl);
    }
    return this;
  }



  /***********************************************************************************************
  Updates the users username
  @param username the new username
  @param oldUsername the old username
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateUsername(String username, String oldUsername,Uri contactUrl){
    if (username != null && oldUsername != null && !username.equals(oldUsername)) {
      content.clear();
      content.put(UsernameData.DATA,username);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the users nickname
  @param nickname the new nickname
  @param oldNickname the old nickname
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateNickname(String nickname,String oldNickname,Uri contactUrl){
    content.clear();
    if (nickname != null && oldNickname != null && !nickname.equals(oldNickname)) {
      content.put(Nickname.NAME,nickname);
    }
    if (content.size() > 0) {
      addUpdateOperation(contactUrl);
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Updates the users gender
  @param gender the new gender
  @param oldGender the old gender
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateGender(String gender,String oldGender,Uri contactUrl){
    if (gender != null && oldGender != null && !gender.equals(oldGender)) {
      content.clear();
      content.put(GenderData.DATA,gender);
      addUpdateOperation(contactUrl);
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Updates the users gaming style
  @param gamingStyle the new gaming style
  @param oldGamingStyle the old gaming style
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateGamingStyle(String gamingStyle,String oldGamingStyle,Uri contactUrl){
    if (gamingStyle != null && oldGamingStyle != null && !gamingStyle.equals(oldGamingStyle)) {
      content.clear();
      content.put(GamingStyleData.DATA,gamingStyle);
      addUpdateOperation(contactUrl);
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Updates the users age
  @param age the new age
  @param oldAge the old age
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateAge(int age, int oldAge,Uri contactUrl){
    if (age != 0 && age != oldAge) {
      content.clear();
      content.put(AgeData.DATA,age);
      addUpdateOperation(contactUrl);
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Updates the users occupation
  @param occupation the new occupation
  @param oldOccupation the old occupation
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateOccupation(String occupation,String oldOccupation,Uri contactUrl){
    if (occupation != null && oldOccupation != null && !occupation.equals(oldOccupation)) {
      content.clear();
      content.put(OccupationData.DATA,occupation);
      addUpdateOperation(contactUrl);
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Updates the users interests
  @param interests the new interests
  @param oldInterests the old interests
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateInterests(String interests,String oldInterests,Uri contactUrl){
    if (interests != null && oldInterests != null && !interests.equals(oldInterests)) {
      content.clear();
      content.put(InterestsData.DATA,interests);
      addUpdateOperation(contactUrl);
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Updates the users biography profile
  @param biography the new biography profile
  @param oldBiography the old biography profile
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateBiography(String biography,String oldBiography,Uri contactUrl){
    if (biography != null && oldBiography != null && !biography.equals(oldBiography)) {
      content.clear();
      content.put(BiographyData.DATA,biography);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the website
  @param website the new website
  @param oldWebsite the old website
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateWebsite(String website,String oldWebsite,Uri contactUrl){
    if (website != null && oldWebsite != null && !website.equals(oldWebsite)) {
      content.clear();
      content.put(WebsiteData.DATA,website);
      addUpdateOperation(contactUrl);
    }
    return this;
  }




  /***********************************************************************************************
  Adds a CALLER_IS_SYNCADAPTER parameter
  ***********************************************************************************************/
  public static Uri addCallerIsSyncAdapterParameter(Uri uri){
    return uri.buildUpon().appendQueryParameter(ContactsContract.CALLER_IS_SYNCADAPTER,"true").build();
  }


  /***********************************************************************************************
  Builds a <code>ContentProviderOperation</code> for a delete operation
  ***********************************************************************************************/
  public static ContentProviderOperation.Builder addDeleteOperation(Uri uri,boolean yield){
    return ContentProviderOperation.newDelete(addCallerIsSyncAdapterParameter(uri)).withYieldAllowed(yield);

  }


  /***********************************************************************************************
  Builds a <code>ContentProviderOperation</code> for an insert operation
   ***********************************************************************************************/
  public static ContentProviderOperation.Builder addInsertOperation(Uri uri,boolean yield){
    return ContentProviderOperation.newInsert(addCallerIsSyncAdapterParameter(uri)).withYieldAllowed(yield);
  }


  /***********************************************************************************************
  Builds a <code>ContentProviderOperation</code> for an update operation
   ***********************************************************************************************/
  public static ContentProviderOperation.Builder addUpdateOperation(Uri uri,boolean yield){
    return ContentProviderOperation.newUpdate(addCallerIsSyncAdapterParameter(uri)).withYieldAllowed(yield);
  }


  /***********************************************************************************************
  Creates a new <code>ContactOperations</code> for adding new contact to the Android Contacts
  @param context the Authenticator Activity context
  @param userId the userId of the sample SyncAdapter user object
  @param accountName the username of the current login
  @param batchOperations any batch operations to perform when creating the new contact 
  @return instance of ContactOperations
  ***********************************************************************************************/
  public static ContactOperations createNewContact(Context context,int userId,String accountName,BatchOperation batchOperations){
    return new ContactOperations(context,userId,accountName,batchOperations);
  }


  /***********************************************************************************************
  Creates a new <code>ContactOperations</code> for adding new group to the Android Contacts
  @param context the Authenticator Activity context
  @param groupId the groupId of the sample SyncAdapter user object
  @param accountName the username of the current login
  @param batchOperations any batch operations to perform when creating the new contact 
  @return instance of ContactOperations
  ***********************************************************************************************/
  public static ContactOperations createNewGroup(Context context,int groupId,String accountName,BatchOperation batchOperations){
    return new ContactOperations(context,groupId,accountName,batchOperations,true);
  }


  /***********************************************************************************************
  Creates a new <code>ContactOperations</code> for updating an existing Android Contact
  @param context the Authenticator Activity context
  @param rawContactId the unique ID of the existing contact
  @param batchOperations any batch operations to perform when updating the contact 
  @return instance of ContactOperations
   ***********************************************************************************************/
  public static ContactOperations updateExistingContact(Context context,long rawContactId,BatchOperation batchOperations){
    return new ContactOperations(context,rawContactId,batchOperations);
  }

}
