////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.xfire;

import java.net.URL;
import org.watto.android.Language;


/***********************************************************************************************
Details about a single XFire friend
***********************************************************************************************/
public class XFireFriend {

  /** The users user name **/
  String username = null;
  /** The users user name **/
  String nickname = null;
  /** The URL to the photo of this user **/
  URL photo = null;
  /** The users gender (m, f) **/
  String gender = null;
  /** The gaming style of the user (newbie, camper, sniper, slayer, etc) **/
  String gamingStyle = null;
  /** The users location **/
  String location = null;
  /** The users country **/
  String country = null;
  /** The age of the user **/
  int age = 0;
  /** The date that the user created their XFire account **/
  String joinDate = null;
  /** The online state of the user (online, offline, etc) **/
  String status = null;
  /** The website of the user **/
  String website = null;
  /** The users real name **/
  String realName = null;
  /** The users occupation **/
  String occupation = null;
  /** The users interests **/
  String interests = null;
  /** The users biography **/
  String biography = null;


  /***********************************************************************************************
  Constructor
  ***********************************************************************************************/
  public XFireFriend(){}


  /***********************************************************************************************
  Creates a new XFire friend with a <i>username</i>
  @param username the username of the friend
  ***********************************************************************************************/
  public XFireFriend(String username){
    this.username = username;
  }


  /***********************************************************************************************
  Gets the age of this user
  @return the <i>age</i>
  ***********************************************************************************************/
  public int getAge(){
    return age;
  }


  /***********************************************************************************************
  Gets the biography of this user
  @return the <i>biography</i>
  ***********************************************************************************************/
  public String getBiography(){
    return biography;
  }


  /***********************************************************************************************
  Gets the country of this user
  @return the <i>country</i>
  ***********************************************************************************************/
  public String getCountry(){
    return country;
  }


  /***********************************************************************************************
  Gets the gaming style of this user
  @return the <i>gamingStyle</i>
  ***********************************************************************************************/
  public String getGamingStyle(){
    return gamingStyle;
  }


  /***********************************************************************************************
  Gets the gender of this user
  @return the <i>gender</i>
  ***********************************************************************************************/
  public String getGender(){
    return gender;
  }


  /***********************************************************************************************
  Gets the interests of this user
  @return the <i>interests</i>
  ***********************************************************************************************/
  public String getInterests(){
    return interests;
  }


  /***********************************************************************************************
  Gets the join date of this user
  @return the <i>joinDate</i>
  ***********************************************************************************************/
  public String getJoinDate(){
    return joinDate;
  }


  /***********************************************************************************************
  Gets the location of this user
  @return the <i>location</i>
  ***********************************************************************************************/
  public String getLocation(){
    return location;
  }


  /***********************************************************************************************
  Gets the nickname of this user
  @return the <i>nickname</i>
  ***********************************************************************************************/
  public String getNickname(){
    return nickname;
  }


  /***********************************************************************************************
  Gets the occupation of this user
  @return the <i>occupation</i>
  ***********************************************************************************************/
  public String getOccupation(){
    return occupation;
  }


  /***********************************************************************************************
  Gets the URL for the photo of this user
  @return the <i>photo</i> URL
  ***********************************************************************************************/
  public URL getPhoto(){
    return photo;
  }


  /***********************************************************************************************
  Gets the real name of this user
  @return the <i>realName</i>
  ***********************************************************************************************/
  public String getRealName(){
    return realName;
  }


  /***********************************************************************************************
  Gets the online status of this user
  @return the <i>status</i>
  ***********************************************************************************************/
  public String getStatus(){
    return status;
  }


  /***********************************************************************************************
  Gets the username of this user
  @return the <i>username</i>
  ***********************************************************************************************/
  public String getUsername(){
    return username;
  }


  /***********************************************************************************************
  Gets the website of this user
  @return the <i>website</i>
  ***********************************************************************************************/
  public String getWebsite(){
    return website;
  }


  /***********************************************************************************************
  Sets the age of this user
  @param age the <i>age</i>
  ***********************************************************************************************/
  public void setAge(int age){
    this.age = age;
  }


  /***********************************************************************************************
  Sets the biography of this user
  @param biography the biography of this user
  ***********************************************************************************************/
  public void setBiography(String biography){
    this.biography = biography;
  }


  /***********************************************************************************************
  Sets the country of this user
  @param country the <i>country</i>
  ***********************************************************************************************/
  public void setCountry(String country){
    
    if (country.equals("au")){
      this.country = Language.get(R.string.country_au);
    }
    else if (country.equals("us")){
      this.country = Language.get(R.string.country_us);
    }
    else if (country.equals("ca")){
      this.country = Language.get(R.string.country_ca);
    }
    else if (country.equals("gb")){
      this.country = Language.get(R.string.country_gb);
    }
    else if (country.equals("fr")){
      this.country = Language.get(R.string.country_fr);
    }
    else {
      this.country = country;
    }
    
  }


  /***********************************************************************************************
  Sets the gaming style of this user
  @param gamingStyle the <i>gamingStyle</i>
  ***********************************************************************************************/
  public void setGamingStyle(String gamingStyle){
    this.gamingStyle = gamingStyle;
  }


  /***********************************************************************************************
  Sets the gender of this user
  @param gender the <i>gender</i>
  ***********************************************************************************************/
  public void setGender(String gender){
    
    if (gender.equals("m")){
      this.gender = Language.get(R.string.gender_male);
    }
    else if (gender.equals("f")){
      this.gender = Language.get(R.string.gender_female);
    }
    else {
      this.gender = null;
    }

  }


  /***********************************************************************************************
  Sets the interests of this user
  @param interests the <i>interests</i>
  ***********************************************************************************************/
  public void setInterests(String interests){
    this.interests = interests;
  }


  /***********************************************************************************************
  Sets the join date of this user
  @param joinDate the join date of this user
  ***********************************************************************************************/
  public void setJoinDate(String joinDate){
    this.joinDate = joinDate;
  }


  /***********************************************************************************************
  Sets the location of this user
  @param location the location of this user
  ***********************************************************************************************/
  public void setLocation(String location){
    this.location = location;
  }


  /***********************************************************************************************
  Sets the nickname of this user
  @param nickname the <i>nickname</i>
  ***********************************************************************************************/
  public void setNickname(String nickname){
    this.nickname = nickname;
  }


  /***********************************************************************************************
  Sets the occupation of this user
  @param occupation the <i>occupation</i>
  ***********************************************************************************************/
  public void setOccupation(String occupation){
    this.occupation = occupation;
  }


  /***********************************************************************************************
  Sets the URL to the photo of this user
  @param photo the URL to the photo of this user
  ***********************************************************************************************/
  public void setPhoto(URL photo){
    this.photo = photo;
  }


  /***********************************************************************************************
  Sets the real name of this user
  @param realName the real name of this user
  ***********************************************************************************************/
  public void setRealName(String realName){
    if (realName != null && realName.equals("null")) {
      realName = null;
    }
    this.realName = realName;
  }


  /***********************************************************************************************
  Sets the online status of this user
  @param status the online status of this user
  ***********************************************************************************************/
  public void setStatus(String status){
    this.status = status;
  }


  /***********************************************************************************************
  Sets the username of this user
  @param username the username of this user
  ***********************************************************************************************/
  public void setUsername(String username){
    if (username != null && username.equals("null")) {
      username = null;
    }
    this.username = username;
  }


  /***********************************************************************************************
  Sets the website of this user
  @param website the <i>website</i>
  ***********************************************************************************************/
  public void setWebsite(String website){
    if (website != null && website.equals("null")) {
      website = null;
    }
    this.website = website;
  }

}
