////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.xfire;

import java.net.URL;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;
import org.watto.android.xml.XMLNode;
import org.watto.android.xml.XMLReader;
import org.watto.program.android.sync.xfire.authenticator.XFireAuthenticator;


/***********************************************************************************************
 Provides utility methods for communicating with the backend server
***********************************************************************************************/
public class XFireQuery {

  /***********************************************************************************************
  Fetches the list of friend data updates from the server
  @return list The list of updates received from the server.
  ***********************************************************************************************/
  public static XFireFriend[] getFriends(){
    try {

      if (!XFireAuthenticator.credentialsKnown()) {
        // The authentication is invalid, so return nothing
        return new XFireFriend[0];
      }

      XMLNode friendsNode = queryServer(XFireAuthenticator.getFriendsUrl());
      if (friendsNode != null && friendsNode.hasChildren()) {

          // Yay, we have friends!
          int friendCount = friendsNode.getChildCount();

          XFireFriend[] friends = new XFireFriend[friendCount];

          // Create the friends
          for (int i = 0;i < friendCount;i++) {
            XMLNode friendNode = friendsNode.getChild(i);
            if (!friendNode.getTag().equals("friend")) {
              // Not a "friend" node for some reason - bad XML!
              continue;
            }

            String username = null;
            try {
              username = friendNode.getChild("username").getContent();
            }
            catch (Throwable t) {
              continue;
            }

            friends[i] = new XFireFriend(username);
          }

          // get further details about each friend
          getFriendsDetails(friends);

          return friends;
        }


      return new XFireFriend[0];
    }
    catch (Throwable t) {
      // An error occurred in the response, or there were parsing errors
      return new XFireFriend[0];
    }
  }


  /***********************************************************************************************
  Gets further details about each friend
  @param friends the friends to get the details for
  ***********************************************************************************************/
  public static void getFriendsDetails(XFireFriend[] friends){
    try {

      int friendCount = friends.length;

      
      for (int i = 0;i < friendCount;i++) {
        XFireFriend friend = friends[i];

        String username = friend.getUsername();

        XMLNode profileNode = queryServer(XFireAuthenticator.getProfileUrl(username));
        if (profileNode != null && profileNode.hasChildren()) {
          
          try {
            friend.setNickname(profileNode.getChild("nickname").getContent());
          }
          catch (Throwable t) {}
          
          try {
            friend.setPhoto(new URL(profileNode.getChild("avatar").getContent()));
          }
          catch (Throwable t) {}
          
          try {
            friend.setGender(profileNode.getChild("gender").getContent());
          }
          catch (Throwable t) {}
          
          try {
            friend.setGamingStyle(profileNode.getChild("gaming_style").getContent());
          }
          catch (Throwable t) {}
          
          try {
            friend.setLocation(profileNode.getChild("location").getContent());
          }
          catch (Throwable t) {}
          
          try {
            friend.setCountry(profileNode.getChild("country").getContent());
          }
          catch (Throwable t) {}
          
          try {
            friend.setAge(Integer.parseInt(profileNode.getChild("age").getContent()));
          }
          catch (Throwable t) {}
          
          try {
            friend.setJoinDate(profileNode.getChild("joindate").getContent());
          }
          catch (Throwable t) {}
          
          try {
            friend.setStatus(profileNode.getChild("status").getContent());
          }
          catch (Throwable t) {}
          
          try {
            friend.setWebsite(profileNode.getChild("website").getContent());
          }
          catch (Throwable t) {}
          
          try {
            friend.setRealName(profileNode.getChild("realname").getContent());
          }
          catch (Throwable t) {}
          
          try {
            friend.setOccupation(profileNode.getChild("occupation").getContent());
          }
          catch (Throwable t) {}
          
          try {
            friend.setInterests(profileNode.getChild("interests").getContent());
          }
          catch (Throwable t) {}
          
          try {
            friend.setBiography(profileNode.getChild("bio").getContent());
          }
          catch (Throwable t) {}


        }

      }

    }
    catch (Throwable t) {
      // An error occurred in the response, or there were parsing errors
    }
  }


  /***********************************************************************************************
  Signs a query, sends it to the server, receives a response and returns it 
  @param queryUrl the URL of the query you wish to perform
  @return the response as an <code>XMLNode</code>, or <b>null</b> if there was an error
  ***********************************************************************************************/
  public static XMLNode queryServer(String queryUrl){
    try {

      HttpGet request = new HttpGet(queryUrl);

      // Send the request
      DefaultHttpClient client = new DefaultHttpClient();
      HttpResponse response = client.execute(request);

      // Get the response code
      if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
        // Success
        String responseMessage = EntityUtils.toString(response.getEntity());

        // Convert the response into an XML tree
        XMLNode xmlTree = XMLReader.read(responseMessage);

        // return the xml response
        return xmlTree;
      }
      return null;
    }
    catch (Throwable t) {
      return null;
    }
  }



}
