////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.xfire.security;

import org.watto.android.Settings;


/***********************************************************************************************
Convenience methods for establishing OAuth 2.0 authentication to XFire
***********************************************************************************************/
public class XFireAuthProtocol {

  /** The username for the user **/
  String username = null;


  /***********************************************************************************************
  Initialises the OAuth provider with the <i>consumerKey</i>, <i>consumerSecret</i> and the 
  <i>callbackUrl</i> for the application
  @param consumerKey the OAuth key for this application
  @param consumerSecret the OAuth secret for this application
  @param callbackUrl the OAuth callback URL for this application
  ***********************************************************************************************/
  public XFireAuthProtocol(){}


  /***********************************************************************************************
  Gets the Custom Url for the user
  @return the <i>customUrl</i>
   ***********************************************************************************************/
  public String getUsername(){
    return username;
  }


  /***********************************************************************************************
  
   ***********************************************************************************************/
  public String getFriendsUrl(){
    return "http://www.xfire.com/xml/" + username + "/user_friends/";
  }


  /***********************************************************************************************
  
  ***********************************************************************************************/
  public String getProfileUrl(String username){
    return "http://www.xfire.com/xml/" + username + "/user_profile/";
  }


  /***********************************************************************************************
  Loads the authentication <i>code</i> and the <i>accessToken</i> from the <code>Settings</code>
  ***********************************************************************************************/
  public void loadFromSettings(){
    // Load the code and access token from the Settings
    if (Settings.hasApplication()) {
      this.username = Settings.getString("XFireUsername");
    }
  }
  
  


  /***********************************************************************************************
  Sets the username for the user
  @param customUrl the <i>customUrl</i>  
   ***********************************************************************************************/
  public void setUsername(String username){
       
    this.username = username;
    Settings.set("XFireUsername",username);
  }

}
